@@ -163,7 +163,7 @@ class rcmail_install
     $value = $this->config[$name];
 
     if ($name == 'des_key' && !$this->configured && !isset($_REQUEST["_$name"]))
-      $value = self::random_key(24);
+      $value = rcube_utils::random_bytes(24);
 
     return $value !== null && $value !== '' ? $value : $default;
   }
@@ -193,7 +193,7 @@ class rcmail_install
 
       // generate new encryption key, never use the default value
       if ($prop == 'des_key' && $value == $this->defaults[$prop])
-        $value = $this->random_key(24);
+        $value = rcube_utils::random_bytes(24);
 
       // convert some form data
       if ($prop == 'debug_level' && !$is_default) {
@@ -569,26 +569,30 @@ class rcmail_install
   * Return a list with available subfolders of the plugins directory
   * (with their associated description in composer.json)
   */
-  function list_plugins() 
+  function list_plugins()
   {
     $plugins = array();
     $plugin_dir = INSTALL_PATH . 'plugins/';
 
-    foreach (glob($plugin_dir . '*') as $path) 
-    {
+    foreach (glob($plugin_dir . '*') as $path) {
+      if (!is_dir($path)) {
+        continue;
+      }
 
-      if (is_dir($path) && is_readable($path.'/composer.json'))
-      {
-        $file_json = json_decode(file_get_contents($path.'/composer.json'));
+      if (is_readable($path.'/composer.json')) {
+        $file_json   = json_decode(file_get_contents($path.'/composer.json'));
         $plugin_desc = $file_json->description ?: 'N/A';
       }
-      else
-      {
+      else {
         $plugin_desc = 'N/A';
       }
 
-      $name = substr($path, strlen($plugin_dir));
-      $plugins[] = array('name' => $name, 'desc' => $plugin_desc, 'enabled' => in_array($name, $this->config['plugins']));
+      $name      = substr($path, strlen($plugin_dir));
+      $plugins[] = array(
+        'name'    => $name,
+        'desc'    => $plugin_desc,
+        'enabled' => in_array($name, (array) $this->config['plugins'])
+      );
     }
 
     return $plugins;
@@ -789,25 +793,4 @@ class rcmail_install
   {
       $this->last_error = $p;
   }
-
-
-  /**
-   * Generarte a ramdom string to be used as encryption key
-   *
-   * @param int Key length
-   * @return string The generated random string
-   * @static
-   */
-  function random_key($length)
-  {
-    $alpha = 'ABCDEFGHIJKLMNOPQERSTUVXYZabcdefghijklmnopqrtsuvwxyz0123456789+*%&?!$-_=';
-    $out = '';
-
-    for ($i=0; $i < $length; $i++)
-      $out .= $alpha{rand(0, strlen($alpha)-1)};
-
-    return $out;
-  }
-
 }
-
