@@ -87,9 +87,10 @@ class rcube_config
         $this->load();
 
         // Defaults, that we do not require you to configure,
-        // but contain information that is used in various
-        // locations in the code:
-        $this->set('contactlist_fields', array('name', 'firstname', 'surname', 'email'));
+        // but contain information that is used in various locations in the code:
+        if (empty($this->prop['contactlist_fields'])) {
+            $this->set('contactlist_fields', array('name', 'firstname', 'surname', 'email'));
+        }
     }
 
     /**
